# -*- coding: utf-8 -*-
##################################################################
#
# (C) Copyright 2006 ObjectRealms, LLC
# All Rights Reserved
#
# This file is part of iterate.
#
# iterate is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# iterate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CMFDeployment; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################
"""
Archetypes speicifc implementation for relation handling
"""

from interfaces import ICheckinCheckoutReference
from interfaces import IIterateAware
from interfaces import IWorkingCopyRelation
from Products.Archetypes import config as atconf
from Products.Archetypes.exceptions import ReferenceException
from Products.Archetypes.ReferenceEngine import Reference
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.component import adapter
from zope.interface import implementer

import logging


logger = logging.getLogger('plone.app.iterate')


@implementer(IWorkingCopyRelation, IAttributeAnnotatable)
class WorkingCopyRelation(Reference):
    """
    Source Object is Working Copy

    Target Object is Baseline Version
    """
    relationship = 'Working Copy Relation'


@implementer(ICheckinCheckoutReference)
@adapter(IIterateAware)
class CheckinCheckoutReferenceAdapter(object):
    """
    default adapter for references.

    on checkout

    forward refs on baseline are copied to wc
    backward refs on baseline are ignored on wc

    on checkin

    forward refs on wc are kept
    backwards refs on wc are kept

    forward refs on baseline get removed
    backward refs on baseline are kept by virtue of UID transferance

    """

    storage_key = 'coci.references'

    def __init__(self, context):
        self.context = context

    def checkout(self, baseline, wc, refs, storage):
        for ref in refs:
            try:
                wc.addReference(ref.targetUID, ref.relationship,
                                referenceClass=ref.__class__)
            except ReferenceException:
                logger.warn(
                    'Reference exception when adding relation %r '
                    'from new working copy %s to uid %s. Ignoring relation.',
                    ref.relationship, '/'.join(wc.getPhysicalPath()),
                    ref.targetUID)

    def checkin(self, *args):
        pass

    checkoutBackReferences = checkinBackReferences = checkin


@implementer(ICheckinCheckoutReference)
class NoCopyReferenceAdapter(object):
    """
    an adapter for references that does not copy them to the wc on checkout.

    additionally custom reference state is kept when the wc is checked in.
    """

    def __init__(self, context):
        self.context = context

    def checkin(self, baseline, wc, refs, storage):
        # move the references from the baseline to the wc

        # one note, on checkin the wc uid is not yet changed to match that of
        # the baseline
        ref_ids = [r.getId() for r in refs]

        baseline_ref_container = getattr(baseline, atconf.REFERENCE_ANNOTATION)
        clipboard = baseline_ref_container.manage_cutObjects(ref_ids)

        wc_ref_container = getattr(wc, atconf.REFERENCE_ANNOTATION)

        # references aren't globally addable w/ associated perm which default
        # copysupport wants to check, temporarily monkey around the issue.
        def _verifyObjectPaste(*args, **kw):
            pass

        wc_ref_container._verifyObjectPaste = _verifyObjectPaste
        try:
            wc_ref_container.manage_pasteObjects(clipboard)
        finally:
            del wc_ref_container._verifyObjectPaste

    def checkout(self, *args):
        pass

    checkoutBackReferences = checkinBackReferences = checkout



# -*- coding: utf-8 -*-
##################################################################
#
# (C) Copyright 2006-2007 ObjectRealms, LLC
# All Rights Reserved
#
# This file is part of iterate.
#
# iterate is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# iterate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CMFDeployment; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################
"""
$Id: workflow.py 1824 2007-02-08 17:59:41Z hazmat $

Applies new checkout specific workflows to content that is checked out.

"""

from Acquisition import aq_base
from plone.app.iterate.interfaces import IIterateSettings
from plone.app.iterate.util import get_storage
from plone.registry.interfaces import IRegistry
from Products.CMFPlacefulWorkflow.PlacefulWorkflowTool import WorkflowPolicyConfig_id  # noqa
from Products.CMFPlacefulWorkflow.WorkflowPolicyConfig import WorkflowPolicyConfig  # noqa
from zope.component import getUtility


USE_WORKFLOW = 'checkout_workflow_policy'

policy_storage = 'previous_wf_policy'


def handleCheckout(event):
    # defer to setting
    registry = getUtility(IRegistry)
    settings = registry.forInterface(IIterateSettings)
    if not settings.enable_checkout_workflow:
        return
    policy_id = str(settings.checkout_workflow_policy)

    existing_policy = getattr(
        aq_base(event.working_copy), WorkflowPolicyConfig_id, None)
    storage = get_storage(event.relation)

    # set config for policy in and below
    policy = WorkflowPolicyConfig(policy_id, policy_id)
    policy.coci_created = True

    if existing_policy is not None:
        storage[policy_storage] = policy

    # we setattr because we want the effect on non containerish objects
    setattr(event.working_copy, WorkflowPolicyConfig_id, policy)
    event.working_copy.notifyWorkflowCreated()
    event.working_copy.reindexObjectSecurity()


def handleCheckin(event):
    policy = getattr(aq_base(event.object), WorkflowPolicyConfig_id, None)
    storage = get_storage(event.relation)
    previous_policy = storage.get(policy_storage)
    if previous_policy is None:
        # only reset workflows we know.. could use are own storage
        if policy and not getattr(policy, 'coci_created', False):
            return
        elif policy is None:
            return
        else:
            delattr(event.object, WorkflowPolicyConfig_id)
    else:
        setattr(event.object, WorkflowPolicyConfig_id, previous_policy)



# -*- coding: utf-8 -*-
from plone.app.iterate.testing import PLONEAPPITERATE_FUNCTIONAL_TESTING
from plone.app.iterate.testing import PLONEAPPITERATEDEX_FUNCTIONAL_TESTING
from plone.testing import layered
from unittest import TestSuite

import doctest


def test_suite():
    suite = TestSuite()
    OPTIONFLAGS = (doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)
    suite.addTest(layered(
        doctest.DocFileSuite(
            'browser.rst',
            optionflags=OPTIONFLAGS,
            package='plone.app.iterate.tests',
        ),
        layer=PLONEAPPITERATE_FUNCTIONAL_TESTING)
    )
    suite.addTest(layered(
        doctest.DocFileSuite(
            'dexterity.rst',
            optionflags=OPTIONFLAGS,
            package='plone.app.iterate.tests',
        ),
        layer=PLONEAPPITERATEDEX_FUNCTIONAL_TESTING)
    )
    return suite



# -*- coding: utf-8 -*-
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.iterate import copier
from plone.app.iterate import interfaces
from plone.app.iterate.dexterity import ITERATE_RELATION_NAME
from plone.app.iterate.dexterity.relation import StagingRelationValue
from plone.app.iterate.event import AfterCheckinEvent
from plone.dexterity.utils import iterSchemata
from Products.CMFCore.utils import getToolByName
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from z3c.relationfield import event
from zc.relation.interfaces import ICatalog
from ZODB.PersistentMapping import PersistentMapping
from zope import component
from zope.annotation.interfaces import IAnnotations
from zope.event import notify
from zope.interface import implementer
from zope.intid.interfaces import IIntIds
from zope.schema import getFieldsInOrder


@implementer(interfaces.IObjectCopier)
class ContentCopier(copier.ContentCopier):

    def copyTo(self, container):
        context = aq_inner(self.context)
        wc = self._copyBaseline(container)
        # get id of objects
        intids = component.getUtility(IIntIds)
        wc_id = intids.getId(wc)
        # create a relation
        relation = StagingRelationValue(wc_id)
        event._setRelation(context, ITERATE_RELATION_NAME, relation)
        #
        self._handleReferences(self.context, wc, 'checkout', relation)
        return wc, relation

    def merge(self):
        baseline = self._getBaseline()

        # delete the working copy reference to the baseline
        wc_ref = self._deleteWorkingCopyRelation()

        # reassemble references on the new baseline
        self._handleReferences(baseline, self.context, 'checkin', wc_ref)

        # move the working copy to the baseline container, deleting the
        # baseline
        new_baseline = self._replaceBaseline(baseline)

        # patch the working copy with baseline info not preserved during
        # checkout
        self._reassembleWorkingCopy(new_baseline, baseline)

        return new_baseline

    def _replaceBaseline(self, baseline):
        wc_id = self.context.getId()
        wc_container = aq_parent(self.context)

        # copy all field values from the working copy to the baseline
        for schema in iterSchemata(baseline):
            for name, field in getFieldsInOrder(schema):
                # Skip read-only fields
                if field.readonly:
                    continue
                if field.__name__ == 'id':
                    continue
                try:
                    value = field.get(schema(self.context))
                except Exception:
                    value = None

                # TODO: We need a way to identify the DCFieldProperty
                # fields and use the appropriate set_name/get_name
                if name == 'effective':
                    baseline.effective_date = self.context.effective()
                elif name == 'expires':
                    baseline.expiration_date = self.context.expires()
                elif name == 'subjects':
                    baseline.setSubject(self.context.Subject())
                else:
                    field.set(baseline, value)

        baseline.reindexObject()

        # copy annotations
        wc_annotations = IAnnotations(self.context)
        baseline_annotations = IAnnotations(baseline)

        baseline_annotations.clear()
        baseline_annotations.update(wc_annotations)

        # delete the working copy
        wc_container._delObject(wc_id)

        return baseline

    def _reassembleWorkingCopy(self, new_baseline, baseline):
        # reattach the source's workflow history, try avoid a dangling ref
        try:
            new_baseline.workflow_history = PersistentMapping(
                baseline.workflow_history.items())
        except AttributeError:
            # No workflow apparently.  Oh well.
            pass

        # reset wf state security directly
        workflow_tool = getToolByName(self.context, 'portal_workflow')
        wfs = workflow_tool.getWorkflowsFor(self.context)
        for wf in wfs:
            if not isinstance(wf, DCWorkflowDefinition):
                continue
            wf.updateRoleMappingsFor(new_baseline)
        return new_baseline

    def _handleReferences(self, baseline, wc, mode, wc_ref):
        pass

    def _deleteWorkingCopyRelation(self):
        # delete the wc reference keeping a reference to it for its annotations
        relation = self._get_relation_to_baseline()
        relation.broken(relation.to_path)
        return relation

    def _get_relation_to_baseline(self):
        context = aq_inner(self.context)
        # get id
        intids = component.getUtility(IIntIds)
        id = intids.getId(context)
        # ask catalog
        catalog = component.getUtility(ICatalog)
        relations = list(catalog.findRelations({'to_id': id}))
        relations = filter(lambda r: r.from_attribute == ITERATE_RELATION_NAME,
                           relations)
        # do we have a baseline in our relations?
        if relations and not len(relations) == 1:
            raise interfaces.CheckinException('Baseline count mismatch')

        if not relations or not relations[0]:
            raise interfaces.CheckinException('Baseline has disappeared')
        return relations[0]

    def _getBaseline(self):
        intids = component.getUtility(IIntIds)
        relation = self._get_relation_to_baseline()
        if relation:
            baseline = intids.getObject(relation.from_id)

        if not baseline:
            raise interfaces.CheckinException('Baseline has disappeared')
        return baseline

    def checkin(self, checkin_message):
        # get the baseline for this working copy, raise if not found
        baseline = self._getBaseline()
        # get a hold of the relation object
        relation = self._get_relation_to_baseline()
        # publish the event for subscribers, early because contexts are about
        # to be manipulated
        notify(event.CheckinEvent(self.context,
                                  baseline,
                                  relation,
                                  checkin_message
                                  ))
        # merge the object back to the baseline with a copier
        copier = component.queryAdapter(self.context,
                                        interfaces.IObjectCopier)
        new_baseline = copier.merge()
        # don't need to unlock the lock disappears with old baseline deletion
        notify(AfterCheckinEvent(new_baseline, checkin_message))
        return new_baseline



# -*- coding: utf-8 -*-
##################################################################
#
# (C) Copyright 2006 ObjectRealms, LLC
# All Rights Reserved
#
# This file is part of iterate.
#
# iterate is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# iterate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CMFDeployment; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################
"""
$Id: interfaces.py 1811 2007-02-06 18:40:02Z hazmat $
"""

from plone.app.iterate import PloneMessageFactory as _
from plone.locking.interfaces import LockType
from plone.locking.interfaces import MAX_TIMEOUT
from zope import schema
from zope.component.interfaces import IObjectEvent
from zope.interface import Attribute
from zope.interface import Interface

import pkg_resources


try:
    pkg_resources.get_distribution('Products.Archetypes')
except pkg_resources.DistributionNotFound:

    class IReference(Interface):
        pass
else:
    from Products.Archetypes.interfaces import IReference


################################
#  Marker interface


class IIterateAware(Interface):
    """An object that can be used for check-in/check-out operations.
    """

#################################
#  Lock types

ITERATE_LOCK = LockType(u'iterate.lock', stealable=False, user_unlockable=False, timeout=MAX_TIMEOUT)  # noqa

#################################
#  Exceptions


class CociException(Exception):
    pass


class CheckinException(CociException):
    pass


class CheckoutException(CociException):
    pass


class ConflictError(CheckinException):
    pass


#################################
# Annotation Key
annotation_key = 'ore.iterate'


class keys(object):
    # various common keys
    checkout_user = 'checkout_user'
    checkout_time = 'checkout_time'


#################################
#  Event Interfaces

class ICheckinEvent(IObjectEvent):
    """ a working copy is being checked in, event.object is the working copy, this
    message is sent before any mutation/merge has been done on the objects
    """

    baseline = Attribute('The Working Copy\'s baseline')
    relation = Attribute('The Working Copy Archetypes Relation Object')
    checkin_message = Attribute('checkin message')


class IAfterCheckinEvent(IObjectEvent):
    """ sent out after an object is checked in """

    checkin_message = Attribute('checkin message')


class IBeforeCheckoutEvent(IObjectEvent):
    """ sent out before a working copy is created """


class ICheckoutEvent(IObjectEvent):
    """ an object is being checked out, event.object is the baseline """

    working_copy = Attribute('The object\'s working copy')
    relation = Attribute('The Working Copy Archetypes Relation Object')


class ICancelCheckoutEvent(IObjectEvent):
    """ a working copy is being cancelled """

    baseline = Attribute('The working copy\'s baseline')


class IWorkingCopyDeletedEvent(IObjectEvent):
    """ a working copy is being deleted, this gets called multiple times at
    different states.
    So on cancel checkout and checkin operations, its mostly designed to
    broadcast an event when the user deletes a working copy using the standard
    container paradigms.
    """

    baseline = Attribute('The working copy baseline')
    relation = Attribute('The Working Copy Archetypes Relation Object')

#################################
# Content Marker Interfaces


class IIterateManagedContent(Interface):
    """Any content managed by iterate - normally a sub-interface is
    applied as a marker to an instance.
    """


class IWorkingCopy(IIterateManagedContent):
    """A working copy/check-out
    """


class IBaseline(IIterateManagedContent):
    """A baseline
    """


class IWorkingCopyRelation(IReference):
    """A relationship to a working copy
    """

#################################
#  Working copy container locator


class IWCContainerLocator(Interface):
    """A named adapter capable of discovering containers where working
    copies can be created.
    """

    available = schema.Bool(
        title=u'Available', description=u'Whether location will be available.')

    title = schema.TextLine(
        title=u'Title', description=u'Title of this location')

    def __call__():
        """Return a container object, or None if available() is False
        """

#################################
#  Interfaces


class ICheckinCheckoutTool(Interface):

    def allowCheckin(content):
        """
        denotes whether a checkin operation can be performed on the content.
        """

    def allowCheckout(content):
        """
        denotes whether a checkout operation can be performed on the content.
        """

    def allowCancelCheckout(content):
        """denotes whether a cancel checkout operation can be performed on the
        content.
        """

    def checkin(content, checkin_messsage):
        """check the working copy in, this will merge the working copy with
        the baseline
        """

    def checkout(container, content):
        pass

    def cancelCheckout(content):
        pass


class IObjectCopier(Interface):
    """ copies and merges the object state
    """

    def copyTo(container):
        """ copy the context to the given container, must also create an AT
        relation using the WorkingCopyRelation.relation name between the
        source and the copy.
        returns the copy.
        """

    def merge():
        """ merge/replace the source with the copy, context is the copy.
        """


class IObjectArchiver(Interface):
    """ iterate needs minimal versioning support
    """

    def save(checkin_message):
        """ save a new version of the object
        """

    def isVersioned(self):
        """ is this content already versioned
        """

    def isVersionable(self):
        """ is versionable check.
        """

    def isModified(self):
        """ is the resource current state, different than its last saved state.
        """


class ICheckinCheckoutPolicy(Interface):
    """Checkin / Checkout Policy"""

    def checkin(checkin_message):
        """checkin the context, if the target has been deleted then raises a
        checkin exception.

       if the object version has changed since the checkout begin (due to
       another checkin) raises a conflict error.
        """

    def checkout(container):
        """
        checkout the content object into the container, iff another object with
        the same id exists the id is amended, the working copy object is
        returned.

        the content object is locked during checkout.

        raises a CheckoutError if the object is already checked out.
        """

    def cancelCheckout():
        """coxtent is a checkout (working copy), this method will go ahead and
        delete
        the working copy.
        """

    def getWorkingCopies():
        pass

    def getBaseline():
        pass

    def getWorkingCopy():
        pass


#################################

class ICheckinCheckoutReference(Interface):
    # a reference processor

    def checkout(baseline, wc, references, storage):
        """
        handle processing of the given references from the baseline
        into the working copy, storage is an annotation for bookkeeping
        information.
        """

    def checkoutBackReferences(baseline, wc, references, storage):
        """
        """

    def checkin(baseline, wc, references, storage):
        """
        """

    def checkinBackReferences(baseline, wc, references, storage):
        """
        """


class IIterateSettings(Interface):

    enable_checkout_workflow = schema.Bool(
        title=_(u'Enable checkout workflow'),
        description=u'',
        default=False,
        required=False
    )

    checkout_workflow_policy = schema.ASCIILine(
        title=_(u'Checkout workflow policy'),
        description=u'',
        default='checkout_workflow_policy',
        required=True
    )



# -*- coding: utf-8 -*-
##################################################################
#
# (C) Copyright 2006 ObjectRealms, LLC
# All Rights Reserved
#
# This file is part of iterate.
#
# iterate is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# iterate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with iterate; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################

from AccessControl import getSecurityManager
from Acquisition import aq_inner
from plone.app.iterate import interfaces
from plone.app.iterate.interfaces import ICheckinCheckoutPolicy
from plone.app.iterate.interfaces import IWorkingCopy
from plone.memoize.view import memoize
from Products.Five.browser import BrowserView

import Products.CMFCore.permissions


class Control(BrowserView):
    """Information about whether iterate can operate.

    This is a public view, referenced in action condition expressions.
    """

    def checkin_allowed(self):
        """Check if a checkin is allowed
        """
        context = aq_inner(self.context)
        checkPermission = getSecurityManager().checkPermission

        if not interfaces.IIterateAware.providedBy(context):
            return False

        archiver = interfaces.IObjectArchiver(context)
        if not archiver.isVersionable():
            return False

        if not IWorkingCopy.providedBy(context):
            return False

        policy = ICheckinCheckoutPolicy(context, None)
        if policy is None:
            return False

        original = policy.getBaseline()
        if original is None:
            return False

        can_modify = checkPermission(
            Products.CMFCore.permissions.ModifyPortalContent,
            original,
        )
        if not can_modify:
            return False

        return True

    def checkout_allowed(self):
        """Check if a checkout is allowed.
        """
        context = aq_inner(self.context)

        if not interfaces.IIterateAware.providedBy(context):
            return False

        archiver = interfaces.IObjectArchiver(context)
        if not archiver.isVersionable():
            return False

        policy = ICheckinCheckoutPolicy(context, None)
        if policy is None:
            return False

        if policy.getWorkingCopy() is not None:
            return False

        # check if its is a checkout
        if policy.getBaseline() is not None:
            return False

        return True

    @memoize
    def cancel_allowed(self):
        """Check to see if the user can cancel the checkout on the
        given working copy
        """
        policy = ICheckinCheckoutPolicy(self.context, None)
        if policy is None:
            return False
        original = policy.getBaseline()
        return original is not None



# -*- coding: utf-8 -*-
##################################################################
#
# (C) Copyright 2006 ObjectRealms, LLC
# All Rights Reserved
#
# This file is part of iterate.
#
# iterate is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# iterate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with iterate; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################

from AccessControl import getSecurityManager
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.iterate import PloneMessageFactory as _
from plone.app.iterate.interfaces import IWCContainerLocator
from Products.CMFCore.interfaces import IDynamicType
from Products.CMFCore.permissions import AddPortalContent
from Products.CMFCore.utils import getToolByName
from zope.component import adapter
from zope.interface import implementer


@implementer(IWCContainerLocator)
@adapter(IDynamicType)
class HomeFolderLocator(object):
    """Locate the current user's home folder, if possible.
    """

    def __init__(self, context):
        self.context = context

    title = _(u'Home folder')

    @property
    def available(self):
        return self() is not None

    def __call__(self):
        return getToolByName(self.context, 'portal_membership').getHomeFolder()


@implementer(IWCContainerLocator)
@adapter(IDynamicType)
class ParentFolderLocator(object):
    """Locate the parent of the context, if the user has the
    Add portal content permission.
    """

    def __init__(self, context):
        self.context = context

    title = _(u'Parent folder')

    @property
    def available(self):
        return bool(
            getSecurityManager().checkPermission(
                AddPortalContent,
                aq_parent(aq_inner(self.context))
            )
        )

    def __call__(self):
        if not self.available:
            return None
        return aq_parent(aq_inner(self.context))



# -*- coding: utf-8 -*-
from plone.app.iterate.base import CheckinCheckoutBasePolicyAdapter
from plone.app.iterate.dexterity.interfaces import IDexterityIterateAware
from plone.app.iterate.dexterity.utils import get_baseline
from plone.app.iterate.dexterity.utils import get_checkout_relation
from plone.app.iterate.dexterity.utils import get_relations
from plone.app.iterate.dexterity.utils import get_working_copy
from plone.app.iterate.event import AfterCheckinEvent
from plone.app.iterate.event import CheckinEvent
from plone.app.iterate.interfaces import CheckinException
from plone.app.iterate.interfaces import IObjectCopier
from plone.app.iterate.util import get_storage
from zope.component import adapter
from zope.component import queryAdapter
from zope.event import notify


@adapter(IDexterityIterateAware)
class CheckinCheckoutPolicyAdapter(CheckinCheckoutBasePolicyAdapter):
    """
    Dexterity Checkin Checkout Policy
    """

    def checkin(self, checkin_message):
        # get the baseline for this working copy, raise if not found
        baseline = self._getBaseline()
        # get a hold of the relation object
        relation = self._get_relation_to_baseline()
        # publish the event for subscribers, early because contexts are about
        # to be manipulated
        notify(CheckinEvent(self.context, baseline, relation, checkin_message))
        # merge the object back to the baseline with a copier
        copier = queryAdapter(self.context, IObjectCopier)
        new_baseline = copier.merge()
        # don't need to unlock the lock disappears with old baseline deletion
        notify(AfterCheckinEvent(new_baseline, checkin_message))
        return new_baseline

    def _get_relation_to_baseline(self):
        # do we have a baseline in our relations?
        relations = get_relations(self.context)
        if relations and not len(relations) == 1:
            raise CheckinException('Baseline count mismatch')
        if not relations or not relations[0]:
            raise CheckinException('Baseline has disappeared')

        return relations[0]

    def _getBaseline(self):
        baseline = get_baseline(self.context)
        if not baseline:
            raise CheckinException('Baseline has disappeared')
        return baseline

    def getBaseline(self):
        return get_baseline(self.context)

    def getWorkingCopy(self):
        return get_working_copy(self.context)

    def getProperties(self, obj, default=None):
        try:
            return get_storage(get_checkout_relation(obj), default=default)
        except AttributeError:
            return default



# -*- coding: utf-8 -*-
from Products.CMFPlone.interfaces import INonInstallable
from zope.interface import implementer


@implementer(INonInstallable)
class HiddenProfiles(object):

    def getNonInstallableProfiles(self):
        """Prevents uninstall profile from showing up in the profile list
        when creating a Plone site.

        """
        return [
            u'plone.app.iterate:uninstall',
            u'plone.app.iterate:plone.app.iterate',
        ]



# -*- coding: utf-8 -*-
##################################################################
#
# (C) Copyright 2006-2007 ObjectRealms, LLC
# (C) Copyright 2007-2017 Plone Foundation
# All Rights Reserved
#
# This file is part of iterate.
#
# iterate is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# iterate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CMFDeployment; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################
"""
Base Checkin Checkout Policy For Content

"""

from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.iterate.event import BeforeCheckoutEvent
from plone.app.iterate.event import CancelCheckoutEvent
from plone.app.iterate.event import CheckoutEvent
from plone.app.iterate.interfaces import ICheckinCheckoutPolicy
from plone.app.iterate.interfaces import IObjectCopier
from plone.app.iterate.util import get_storage
from zope.component import queryAdapter
from zope.event import notify
from zope.interface import implementer


@implementer(ICheckinCheckoutPolicy)
class CheckinCheckoutBasePolicyAdapter(object):
    """Base Checkin Checkout Policy For Content

    - on checkout context is the baseline
    - on checkin context is the working copy.

    """

    # used when creating baseline version for first time
    default_base_message = 'Created Baseline'

    def __init__(self, context):
        self.context = context

    def checkin(self, checkin_message):
        raise NotImplemented()

    def checkout(self, container):
        # see interface
        notify(BeforeCheckoutEvent(self.context))

        # use the object copier to checkout the content to the container
        copier = queryAdapter(self.context, IObjectCopier)
        working_copy, relation = copier.copyTo(container)

        # publish the event for any subscribers
        notify(CheckoutEvent(self.context, working_copy, relation))

        # finally return the working copy
        return working_copy

    def cancelCheckout(self):
        # see interface

        # get the baseline
        baseline = self._getBaseline()

        # publish an event
        notify(CancelCheckoutEvent(self.context, baseline))

        # delete the working copy
        wc_container = aq_parent(aq_inner(self.context))
        wc_container.manage_delObjects([self.context.getId()])

        return baseline

    #################################
    #  Checkin Support Methods

    def getBaseline(self):
        raise NotImplemented()

    def getWorkingCopy(self):
        raise NotImplemented()

    def getProperties(self, obj, default=None):
        return get_storage(obj, default=default)



# -*- coding: utf-8 -*-
from plone.app.iterate.interfaces import IBaseline
from plone.app.iterate.interfaces import ICheckinCheckoutPolicy
from plone.app.iterate.interfaces import IIterateAware
from plone.app.iterate.interfaces import IWorkingCopy
from plone.app.iterate.testing import PLONEAPPITERATEDEX_INTEGRATION_TESTING
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.dexterity.utils import createContentInContainer
from unittest import TestCase


class TestObjectsProvideCorrectInterfaces(TestCase):
    """Since p.a.iterate replaces the baseline on checkin with the working copy
    but p.a.stagingbehavior just copies the values, the provided interfaces
    may be wrong after checkin.

    For making sure that provided interfaces are correct in every state we
    test it here.

    See: https://dev.plone.org/ticket/13163
    """

    layer = PLONEAPPITERATEDEX_INTEGRATION_TESTING

    def setUp(self):
        super(TestObjectsProvideCorrectInterfaces, self).setUp()

        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])
        login(self.portal, TEST_USER_NAME)

        # create a folder where everything of this test suite should happen
        self.assertNotIn('test-folder', self.portal.objectIds())
        self.folder = self.portal.get(
            self.portal.invokeFactory('Folder', 'test-folder'))

        self.obj = createContentInContainer(self.folder, 'Document')

    def tearDown(self):
        self.portal.manage_delObjects([self.folder.id])
        logout()
        setRoles(self.portal, TEST_USER_ID, ['Member'])
        super(TestObjectsProvideCorrectInterfaces, self).tearDown()

    def do_checkout(self):
        policy = ICheckinCheckoutPolicy(self.obj)
        working_copy = policy.checkout(self.folder)
        return working_copy

    def do_cancel(self, working_copy):
        policy = ICheckinCheckoutPolicy(working_copy)
        policy.cancelCheckout()

    def do_checkin(self, working_copy):
        policy = ICheckinCheckoutPolicy(working_copy)
        policy.checkin('')

    def test_before_checkout(self):
        self.assertTrue(self.obj)
        self.assertTrue(IIterateAware.providedBy(self.obj))
        self.assertFalse(IBaseline.providedBy(self.obj))
        self.assertFalse(IWorkingCopy.providedBy(self.obj))

    def test_after_checkout(self):
        working_copy = self.do_checkout()
        self.assertTrue(working_copy)
        self.assertTrue(IIterateAware.providedBy(working_copy))
        self.assertFalse(IBaseline.providedBy(working_copy))
        self.assertTrue(IWorkingCopy.providedBy(working_copy))

        self.assertTrue(IIterateAware.providedBy(self.obj))
        self.assertTrue(IBaseline.providedBy(self.obj))
        self.assertFalse(IWorkingCopy.providedBy(self.obj))

    def test_after_cancel_checkout(self):
        working_copy = self.do_checkout()
        self.assertTrue(working_copy)

        self.do_cancel(working_copy)
        self.assertTrue(IIterateAware.providedBy(self.obj))
        self.assertFalse(IBaseline.providedBy(self.obj))
        self.assertFalse(IWorkingCopy.providedBy(self.obj))

    def test_after_checkin(self):
        working_copy = self.do_checkout()
        self.assertTrue(working_copy)

        self.do_checkin(working_copy)
        self.assertTrue(IIterateAware.providedBy(self.obj))
        self.assertFalse(IBaseline.providedBy(self.obj))
        self.assertFalse(IWorkingCopy.providedBy(self.obj))



# -*- coding: utf-8 -*-
##################################################################
#
# (C) Copyright 2006 ObjectRealms, LLC
# All Rights Reserved
#
# This file is part of iterate.
#
# iterate is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# iterate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with iterate; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################
"""
"""

from plone.app.iterate import permissions  # noqa
from zope.i18nmessageid import MessageFactory

import logging


PloneMessageFactory = MessageFactory('plone')
logger = logging.getLogger('plone.app.iterate')


try:
    import plone.app.relationfield  # noqa
except ImportError:
    logger.warn('Dexterity support for iterate is not available. '
                'You must install plone.app.relationfield')


try:
    import plone.app.stagingbehavior  # noqa
    logger.error(
        'plone.app.stagingbehavior should NOT be installed with this version '
        'of plone.app.iterate. You may experience problems running this '
        'configuration. plone.app.iterate now has dexterity suport built-in.'
    )
except ImportError:
    pass



# -*- coding: utf-8 -*-
##################################################################
#
# (C) Copyright 2006-2007 ObjectRealms, LLC
# All Rights Reserved
#
# This file is part of iterate.
#
# iterate is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# iterate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CMFDeployment; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################
"""
Archetypes Checkin Checkout Policy For Content

"""
from plone.app.iterate.base import CheckinCheckoutBasePolicyAdapter
from plone.app.iterate.event import AfterCheckinEvent
from plone.app.iterate.event import CheckinEvent
from plone.app.iterate.interfaces import CheckinException
from plone.app.iterate.interfaces import IIterateAware
from plone.app.iterate.interfaces import IObjectCopier
from plone.app.iterate.relation import WorkingCopyRelation
from plone.app.iterate.util import get_storage
from Products.Archetypes.interfaces import IReferenceable
from zope.component import adapter
from zope.component import queryAdapter
from zope.event import notify


@adapter(IIterateAware)
class CheckinCheckoutPolicyAdapter(CheckinCheckoutBasePolicyAdapter):
    """Checkin Checkout Policy For Archetypes Content

    - on checkout context is the baseline
    - on checkin context is the working copy.
    """

    def checkin(self, checkin_message):
        # see interface

        # get the baseline for this working copy, raise if not found
        baseline = self._getBaseline()

        # get a hold of the relation object
        wc_ref = self.context.getReferenceImpl(
            WorkingCopyRelation.relationship
        )[0]

        # publish the event for subscribers, early because contexts are about
        # to be manipulated
        notify(CheckinEvent(self.context, baseline, wc_ref, checkin_message))

        # merge the object back to the baseline with a copier

        # XXX by gotcha
        # bug we should or use a getAdapter call or test if copier is None
        copier = queryAdapter(self.context, IObjectCopier)
        new_baseline = copier.merge()

        # don't need to unlock the lock disappears with old baseline deletion
        notify(AfterCheckinEvent(new_baseline, checkin_message))

        return new_baseline

    #################################
    #  Checkin Support Methods

    def _getBaseline(self):
        # follow the working copy's reference back to the baseline
        refs = self.context.getReferences(WorkingCopyRelation.relationship)

        if not len(refs) == 1:
            raise CheckinException('Baseline count mismatch')

        if not refs or refs[0] is None:
            raise CheckinException('Baseline has disappeared')

        baseline = refs[0]
        return baseline

    def getBaseline(self):
        if IReferenceable.providedBy(self.context):
            refs = self.context.getReferences(WorkingCopyRelation.relationship)
            if refs:
                return refs[0]

    def getWorkingCopy(self):
        if IReferenceable.providedBy(self.context):
            refs = self.context.getBRefs(WorkingCopyRelation.relationship)
            if refs:
                return refs[0]

    def getProperties(self, obj, default=None):
        return get_storage(obj, default=default)



# -*- coding: utf-8 -*-
"""Deprecation warning for plone.app.iterate:plone.app.iterate profile."""

import warnings


def deprecate_profile(tool):
    """Deprecation profile plone.app.iterate."""
    warnings.warn(
        'The profile with id "plone.app.iterate" was renamed to "default".',
        DeprecationWarning
    )



# -*- coding: utf-8 -*-
"""
$Id: diff.py 1807 2007-02-06 06:52:46Z hazmat $
"""

from plone.app.iterate.interfaces import IBaseline
from plone.app.iterate.interfaces import ICheckinCheckoutPolicy
from plone.app.iterate.interfaces import IWorkingCopy
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView


class DiffView(BrowserView):

    def __call__(self):
        policy = ICheckinCheckoutPolicy(self.context)
        if IBaseline.providedBy(self.context):
            self.baseline = self.context
            self.working_copy = policy.getWorkingCopy()
        elif IWorkingCopy.providedBy(self.context):
            self.working_copy = self.context
            self.baseline = policy.getBaseline()
        else:
            raise AttributeError('Invalid Context')
        return self.index()

    def diffs(self):
        diff = getToolByName(self.context, 'portal_diff')
        return diff.createChangeSet(self.baseline,
                                    self.working_copy,
                                    id1='Baseline',
                                    id2='Working Copy')



# -*- coding: utf-8 -*-
"""Setup plone.app.iterate."""
from setuptools import find_packages
from setuptools import setup


LONG_DESCRIPTION = (
    open('README.rst').read() +
    '\n' +
    open('CHANGES.rst').read() +
    '\n')

setup(
    name='plone.app.iterate',
    version='3.3.2',
    description='check-out/check-in staging for Plone',
    long_description=LONG_DESCRIPTION,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Plone',
        'Framework :: Plone :: 5.0',
        'Framework :: Plone :: 5.1',
        'Framework :: Zope2',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
    ],
    keywords='check-out check-in staging',
    author='Plone Foundation',
    author_email='plone-developers@lists.sourceforge.net',
    url='https://pypi.python.org/pypi/plone.app.iterate',
    license='GPL version 2',
    packages=find_packages(exclude=['ez_setup']),
    namespace_packages=['plone', 'plone.app'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'Acquisition',
        'DateTime',
        'plone.locking',
        'plone.memoize',
        'Products.Archetypes',
        'Products.CMFCore',
        'Products.CMFEditions',
        'Products.CMFPlacefulWorkflow',
        'Products.DCWorkflow',
        'Products.GenericSetup>=1.8.2',
        'Products.statusmessages',
        'setuptools',
        'ZODB3',
        'zope.annotation',
        'zope.component',
        'zope.event',
        'zope.i18nmessageid',
        'zope.interface',
        'zope.lifecycleevent',
        'zope.schema',
        'zope.viewlet',
    ],
    extras_require={
        'archetypes': [
            'Products.Archetypes',
        ],
        'test': [
            'plone.app.testing',
            'plone.app.contenttypes',
            'plone.app.robotframework',
            'Products.Archetypes',
        ]
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)



# -*- coding: utf-8 -*-
##################################################################
#
# (C) Copyright 2006-2007 ObjectRealms, LLC
# All Rights Reserved
#
# This file is part of iterate.
#
# iterate is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# iterate is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with CMFDeployment; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##################################################################

"""
Archtypes specific copier, dexterity folder has its own!
"""

from Acquisition import aq_base
from Acquisition import aq_inner
from Acquisition import aq_parent
from interfaces import CheckinException
from Products.Archetypes.Referenceable import Referenceable
from Products.CMFCore import interfaces as cmf_ifaces
from Products.CMFCore.utils import getToolByName
from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
from relation import WorkingCopyRelation
from ZODB.PersistentMapping import PersistentMapping
from zope import component
from zope import interface
from zope.annotation.interfaces import IAnnotations
from zope.event import notify
from zope.lifecycleevent import ObjectMovedEvent

import interfaces


@interface.implementer(interfaces.IObjectCopier)
@component.adapter(interfaces.IIterateAware)
class ContentCopier(object):

    def __init__(self, context):
        self.context = context

    def copyTo(self, container):
        wc = self._copyBaseline(container)
        wc_ref = wc.addReference(
            self.context,
            relationship=WorkingCopyRelation.relationship,
            referenceClass=WorkingCopyRelation)
        self._handleReferences(self.context, wc, 'checkout', wc_ref)
        return wc, wc_ref

    def merge(self):
        baseline = self._getBaseline()

        # delete the working copy reference to the baseline
        wc_ref = self._deleteWorkingCopyRelation()

        # reassemble references on the new baseline
        self._handleReferences(baseline, self.context, 'checkin', wc_ref)

        # move the working copy to the baseline container, deleting
        # the baseline
        new_baseline = self._replaceBaseline(baseline)

        # patch the working copy with baseline info not preserved
        # during checkout
        self._reassembleWorkingCopy(new_baseline, baseline)

        return new_baseline

    def _getBaseline(self):
        # follow the working copy's reference back to the baseline
        refs = self.context.getRefs(WorkingCopyRelation.relationship)

        if not len(refs) == 1:
            raise CheckinException('Baseline count mismatch')

        if not refs or refs[0] is None:
            raise CheckinException('Baseline has disappeared')

        baseline = refs[0]
        return baseline

    def _replaceBaseline(self, baseline):
        # move the working copy object to the baseline, returns the
        # new baseline
        baseline_id = baseline.getId()

        # delete the baseline from the folder to make room for the
        # committed working copy
        baseline_container = aq_parent(aq_inner(baseline))
        # Check if we are a default_page, because this property of the
        # container might get lost.
        is_default_page = (
            baseline_container.getProperty('default_page', '') == baseline_id)
        baseline_pos = baseline_container.getObjectPosition(baseline_id)
        baseline_container._delOb(baseline_id)

        # uninedxing the deleted baseline object from portal_catalog
        portal_catalog = getToolByName(self.context, 'portal_catalog')
        portal_catalog.unindexObject(baseline)

        # delete the working copy from the its container
        wc_container = aq_parent(aq_inner(self.context))

        # trick out the at machinery to not delete references
        self.context._v_cp_refs = 1
        self.context._v_is_cp = 0

        wc_id = self.context.getId()
        wc_container.manage_delObjects([wc_id])

        # move the working copy back to the baseline container
        working_copy = aq_base(self.context)
        working_copy.setId(baseline_id)
        baseline_container._setOb(baseline_id, working_copy)
        baseline_container.moveObjectToPosition(baseline_id, baseline_pos)

        new_baseline = baseline_container._getOb(baseline_id)
        if is_default_page:
            # Restore default_page to container.  Note that the property might
            # have been removed by an event handler in the mean time.
            if baseline_container.hasProperty('default_page'):
                baseline_container._updateProperty('default_page', baseline_id)
            else:
                baseline_container._setProperty('default_page', baseline_id)

        # reregister our references with the reference machinery after moving
        Referenceable.manage_afterAdd(new_baseline, new_baseline,
                                      baseline_container)

        notify(ObjectMovedEvent(new_baseline, wc_container,
                                wc_id, baseline_container, baseline_id))

        return new_baseline

    def _reassembleWorkingCopy(self, new_baseline, baseline):
        # reattach the source's workflow history, try avoid a dangling ref
        try:
            new_baseline.workflow_history = PersistentMapping(
                baseline.workflow_history.items())
        except AttributeError:
            # No workflow apparently.  Oh well.
            pass

        # reset wf state security directly
        workflow_tool = getToolByName(self.context, 'portal_workflow')
        wfs = workflow_tool.getWorkflowsFor(self.context)
        for wf in wfs:
            if not isinstance(wf, DCWorkflowDefinition):
                continue
            wf.updateRoleMappingsFor(new_baseline)

        # Reattach the source's uid, this will update wc refs to point
        # back to the new baseline.  This may introduce duplicate
        # references, so we check that and fix them.
        self._recursivelyReattachUIDs(baseline, new_baseline)

        # reattach the source's history id, to get the previous
        # version ancestry
        histid_handler = getToolByName(self.context, 'portal_historyidhandler')
        huid = histid_handler.getUid(baseline)
        histid_handler.setUid(new_baseline, huid, check_uniqueness=False)

        return new_baseline

    def _recursivelyReattachUIDs(self, baseline, new_baseline):
        original_refs = len(new_baseline.getRefs())
        original_back_refs = len(new_baseline.getBRefs())
        new_baseline._setUID(baseline.UID())
        new_refs = len(new_baseline.getRefs())
        new_back_refs = len(new_baseline.getBRefs())
        if original_refs != new_refs:
            self._removeDuplicateReferences(new_baseline, backrefs=False)
        if original_back_refs != new_back_refs:
            self._removeDuplicateReferences(new_baseline, backrefs=True)

        if cmf_ifaces.IFolderish.providedBy(baseline):
            new_ids = new_baseline.contentIds()
            for child in baseline.contentValues():
                if child.getId() in new_ids:
                    self._recursivelyReattachUIDs(
                        child, new_baseline[child.getId()])

    def _removeDuplicateReferences(self, item, backrefs=False):
        # Remove duplicate (back) references from this item.
        reference_tool = getToolByName(self.context, 'reference_catalog')
        if backrefs:
            ref_func = reference_tool.getBackReferences
        else:
            ref_func = reference_tool.getReferences
        try:
            # Plone 4.1 or later
            brains = ref_func(item, objects=False)
        except TypeError:
            # Plone 4.0 or earlier.  Nothing to fix here
            return
        for brain in brains:
            if brain.getObject() is None:
                reference_tool.uncatalog_object(brain.getPath())

    def _deleteWorkingCopyRelation(self):
        # delete the wc reference keeping a reference to it for its annotations
        refs = self.context.getReferenceImpl(WorkingCopyRelation.relationship)
        wc_ref = refs[0]
        self.context.deleteReferences(WorkingCopyRelation.relationship)
        return wc_ref

    #################################
    # Checkout Support Methods

    def _copyBaseline(self, container):
        # copy the context from source to the target container
        source_container = aq_parent(aq_inner(self.context))
        clipboard = source_container.manage_copyObjects([self.context.getId()])
        result = container.manage_pasteObjects(clipboard)

        # get a reference to the working copy
        target_id = result[0]['new_id']
        target = container._getOb(target_id)
        return target

    def _handleReferences(self, baseline, wc, mode, wc_ref):

        annotations = IAnnotations(wc_ref)

        baseline_adapter = interfaces.ICheckinCheckoutReference(baseline)

        # handle forward references
        for relationship in baseline.getRelationships():
            # look for a named relation adapter first
            adapter = component.queryAdapter(
                baseline, interfaces.ICheckinCheckoutReference, relationship)

            if adapter is None:  # default
                adapter = baseline_adapter

            references = baseline.getReferenceImpl(relationship)

            mode_method = getattr(adapter, mode)
            mode_method(baseline, wc, references, annotations)

        mode = mode + 'BackReferences'

        # handle backward references
        for relationship in baseline.getBRelationships():
            adapter = component.queryAdapter(
                baseline, interfaces.ICheckinCheckoutReference, relationship)
            if adapter is None:
                adapter = baseline_adapter

            references = baseline.getBackReferenceImpl(relationship)

            mode_method = getattr(adapter, mode)
            mode_method(baseline, wc, references, annotations)



# -*- coding: utf-8 -*-
from plone.app.folder.folder import ATFolder
from plone.app.folder.folder import ATFolderSchema
from Products.Archetypes import atapi
from Products.ATContentTypes.content.base import registerATCT


RichFolderSchema = ATFolderSchema.copy() + atapi.Schema((
    atapi.TextField('text',
                    default_output_type='text/x-html-safe',
                    widget=atapi.RichWidget(),
                    ),
))


class RichFolder(ATFolder):
    """ sample content type for testing purposes """

    schema = RichFolderSchema
    portal_type = 'RichFolder'


registerATCT(RichFolder, 'plone.app.iterate')


def addRichFolder(container, id, **kwargs):
    """ at-constructor copied from ClassGen.py """
    obj = RichFolder(id)
    container._setObject(id, obj, suppress_events=True)
    obj = container._getOb(id)
    obj.manage_afterAdd(obj, container)
    obj.initializeArchetype(**kwargs)
    return obj.getId()
